@@ -1,19 +1,24 @@
 from . import by as _by
+from parameters import parameters
+import os
 
-class PitchTier(object):
-    """A PitchTier."""
+class Tier(object):
+    """An abstract Tier."""
     
-    __slots__ = ['__objects', '__start_time', '__end_time', '__objects_to_sort', '_indexed']
+    __slots__ = ['__points', '__start_time', '__end_time', '__points_to_sort', '_indexed', '__original_dir']
 
     
-    def __init__(self, start_time, end_time):
-        self.__objects = []
+    def __init__(self, start_time, end_time, name='', points=None, original_dir=''):
+        self.__points = [points, []][points==None]
         self.__start_time = start_time
         self.__end_time = end_time
+        self.__original_dir = original_dir
         self._indexed = True
+        self.name = name
+        self.tier_type = None
 
     def add_point(self, point):
-        """Adds point to the PitchTier."""
+        """Adds point to the Tier."""
         point = Point(point.time, point.value, superior_PitchTier=self)
         
         if point.time < self.start_time or point.time > self.end_time:
@@ -21,26 +26,25 @@ class PitchTier(object):
         
         time = point.time
         
-        for this_point in self.__objects:
+        for this_point in self.__points:
             if this_point.time==time:
                 raise Exception(f'Point object with time {time} already exists.')
         
-        self.__objects.append(point)
+        self.__points.append(point)
         
-        self.__objects_to_sort = [(point.time, point) for point in self.__objects]
-        self.__objects = [i[1] for i in sorted(self.__objects_to_sort, key=lambda x: x[0])]
+        self.__points_to_sort = [(point.time, point) for point in self.__points]
+        self.__points = [i[1] for i in sorted(self.__points_to_sort, key=lambda x: x[0])]
         
         self._indexed = False
         self._reindex()
         self._indexed = True
     
-    def to_plot(self, start_time=None, end_time=None, size=None):
-        """Plots the points of the PitchTier from between two time points. If the time points are not specified, start time and end time are used."""
+    def to_plot(self, start_time=None, end_time=None):
+        """Plots the points of the Tier from between two time points. If the time points are not specified, start time and end time are used. To change related parameters, use parameters.parameters."""
         import seaborn as sns
         from matplotlib import pyplot as plt
 
-        if size!=None:
-            plt.figure(figsize=size)
+        plt.figure(figsize=parameters['to_plot']['size'])
         
         if start_time==None:
             start_time = self.start_time
@@ -55,8 +59,8 @@ class PitchTier(object):
         
         return plot
     
-    def get_point(self, by_target, by, ignore_missing_point=False):
-        """Get Point of the PitchTier with chosen method."""
+    def get_point(self, by_target, by):
+        """Get Point of the Tier with chosen method. To change related parameters, use parameters.parameters."""
         
         try:
             by.by_method
@@ -64,51 +68,51 @@ class PitchTier(object):
             raise Exception(f'Invalid by_method. Use help() to see valid methods.')
         
         if by.by_method=='by_time':
-            for point in self.__objects:
+            for point in self.__points:
                 if point.time==by_target:
                     return point
 
         elif by.by_method=='by_index':
-            point = self.__objects[by_target-1]
+            point = self.__points[by_target-1]
             
             return point
 
         elif by.by_method=='by_point':
-            for point in self.__objects:
+            for point in self.__points:
                 if point==by_target:
                     return point
         
-        if not ignore_missing_point:
+        if not parameters['points']['ignore_missing_point']:
             raise Exception('Point not found.')
         
-    def remove_point(self, by_target, by, ignore_missing_point=False):
-        """Remove Point of the PitchTier with chosen method."""
+    def remove_point(self, by_target, by):
+        """Remove Point of the Tier with chosen method. To change related parameters, use parameters.parameters."""
         
         try:
             by.by_method
         except AttributeError:
             raise Exception(f'Invalid by_method. Use help() to see valid methods.')
         
-        point = self.get_point(by_target, by, ignore_missing_point=ignore_missing_point)
+        point = self.get_point(by_target, by)
         
         if point==None:
             pass
         else:
-            self.__objects.remove(point)
+            self.__points.remove(point)
         
         self._indexed = False
         self._reindex()
         self._indexed = True
     
-    def shift_point_to_time(self, target_time, by_target, by, ignore_missing_point=False):
-        """Shift Point of the PitchTier to chosen time with chosen method."""
+    def shift_point_to_time(self, target_time, by_target, by):
+        """Shift Point of the Tier to chosen time with chosen method. To change related parameters, use parameters.parameters."""
 
         try:
             by.by_method
         except AttributeError:
             raise Exception(f'Invalid by_method. Use help() to see valid methods.')
         
-        point = self.get_point(by_target, by, ignore_missing_point=ignore_missing_point)
+        point = self.get_point(by_target, by)
         
         if point==None:
             pass
@@ -121,8 +125,8 @@ class PitchTier(object):
             
             self.add_point(point)
     
-    def batch_raise_value(self, by_target, by, value, ignore_missing_point=False):
-        """Batch raise or lower value of points of the PitchTier."""
+    def batch_raise_value(self, by_target, by, value):
+        """Batch raise or lower value of points of the Tier. To change related parameters, use parameters.parameters."""
         points = []
         
         try:
@@ -132,9 +136,9 @@ class PitchTier(object):
         
         if by.by_method=='by_point':
             for point in by_target:
-                if point in self.__objects:
+                if point in self.__points:
                     points.append(point)
-                elif ignore_missing_point:
+                elif parameters['points']['ignore_missing_point']:
                     pass
                 else:
                     raise Exception('Point not found.')
@@ -144,7 +148,7 @@ class PitchTier(object):
                 points.append(self.get_point(index, _by.Index()))
                 
         elif by.by_method=='by_time':
-            for point in self.__objects:
+            for point in self.__points:
                 if by_target[0] <= point.time <= by_target[1]:
                     points.append(point)
         
@@ -152,11 +156,11 @@ class PitchTier(object):
             point.value+=value
     
     def to_dataframe(self):
-        """Convert the PitchTier to dataframe."""
+        """Convert the Tier to dataframe."""
         import pandas as pd
         
         df = pd.DataFrame()
-        for point in self.__objects:
+        for point in self.__points:
             point_index, time, value = point.point_index, point.time, point.value
             
             df = df.append(
@@ -168,22 +172,36 @@ class PitchTier(object):
             )
         return df
     
-    def write_to_file(self, file):
-        """Write the PitchTier to file."""
+    def write_to_file(self, file=''):
+        """Write the Tier to file. By default saves to the original directory, if any. To change related parameters, use parameters.parameters."""
         
+        if file=='':
+            file = f'{self.original_dir}/{self.name}.{self.tier_type}'
+        
+        if os.path.isfile(file):
+            if not parameters['write_to_file']['replace']:
+                if parameters['write_to_file']['duplicate']:
+                    original_dir = os.path.dirname(file)
+                    file_name = f"{os.path.basename(file).split('.')[0]} {parameters['write_to_file']['duplicated name extension']}"
+                    file_extension = os.path.basename(file).split('.')[1]
+                    
+                    file = f'{original_dir}/{file_name}.{file_extension}'
+                else:
+                    raise Exception(f'{file} already exists. If you wish to replace it, set \'replace\' to True. If you wish to duplicate it, set \'duplicate\' to True.')
+
         with open(file, 'w') as f:
             f.writelines(
                 [
                     'File type = "ooTextFile"\n',
-                    'Object class = "PitchTier"\n',
+                    f'Object class = "{self.tier_type}"\n',
                     '\n',
                     f'xmin = {self.start_time} \n',
                     f'xmax = {self.end_time} \n',
-                    f'points: size = {len(self.__objects)} \n'
+                    f'points: size = {len(self.__points)} \n'
                 ]
             )
             
-            for point in self.__objects:
+            for point in self.__points:
                 f.writelines(
                     [
                         f'points [{point.point_index}]:\n'
@@ -199,30 +217,27 @@ class PitchTier(object):
         if point.time < self.start_time or point.time > self.end_time:
             raise Exception('Trying to set a point outside of time linits.')
         
-        time = point.time      
-        new_objects = []
-        old_objects = self.objects.copy()
-        appended = False
+        time = point.time
         
-        for this_point in self.__objects:
+        for this_point in self.__points:
             if this_point.time==time:
                 raise Exception(f'Point object with time {time} already exists.')
         
-        self.__objects.append(point)
+        self.__points.append(point)
     
     def _reindex(self):
-        for idx, point in enumerate(self.__objects):
+        for idx, point in enumerate(self.__points):
             point.point_index = idx+1
 
     @property
-    def objects(self):
-        """Points of this PitchTier."""
+    def points(self):
+        """Points of this Tier."""
         
-        return self.__objects
+        return self.__points
         
     @property
     def start_time(self):
-        """Start time of this PitchTier."""
+        """Start time of this Tier."""
 
         return self.__start_time
 
@@ -234,7 +249,7 @@ class PitchTier(object):
 
     @property
     def end_time(self):
-        """End time of this PitchTier."""
+        """End time of this Tier."""
 
         return self.__end_time
 
@@ -246,14 +261,36 @@ class PitchTier(object):
     
     @property
     def indexed(self):
+        
         return self._indexed
+    
+    @property
+    def original_dir(self):
+        
+        return self.__original_dir
         
     def __getitem__(self, item):
-        return self.objects[item]
+        return self.points[item]
 
     def __repr__(self):
-        return f'{self.__class__.__name__}(start_time={self.start_time}, end_time={self.end_time}, duration={self.end_time-self.start_time}, points={self.objects})'
+        return f'{self.__class__.__name__}(name={self.name}, start_time={self.start_time}, end_time={self.end_time}, duration={self.end_time-self.start_time}, points={self.points})'
+
+class PitchTier(Tier):
+    """A PitchTier."""
     
+    def __init__(self, start_time=0, end_time=0, name='', points=None, original_dir=''):
+        super(PitchTier, self).__init__(start_time, end_time, name, points, original_dir)
+        
+        self.tier_type = 'PitchTier'
+
+class DurationTier(Tier):
+    """A DurationTier."""
+    
+    def __init__(self, start_time=0, end_time=0, name='', points=None, original_dir=''):
+        super(DurationTier, self).__init__(start_time, end_time, name, points, original_dir)
+        
+        self.tier_type = 'DurationTier'
+
 class Point(object):
     
     __slots__ = ['__time', '__value', '__point_index', '__superior_PitchTier']
@@ -261,7 +298,7 @@ class Point(object):
     def __init__(self, time, value, point_index=None, superior_PitchTier=None):
 
         if value < 0:
-            raise Exception('Invalid pitch value.')
+            raise Exception('Invalid value.')
         if time < 0:
             raise Exception('Invalid time.')
         if point_index!=None:
